//+------------------------------------------------------------------+
//|                                                      Schnick.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//|     Support Vector
//|                     Machine Learning Tool
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|     ,  
//| Support Vector Machine Tool 'svMachineTool.ex5'
//+------------------------------------------------------------------+
#import "svMachineTool.ex5"
enum ENUM_TRADE {BUY,SELL};
enum ENUM_OPTION {OP_MEMORY,OP_MAXCYCLES,OP_TOLERANCE};
int  initSVMachine(void);
void setIndicatorHandles(int handle,int &indicatorHandles[],int offset,int N);
void setParameter(int handle,ENUM_OPTION option,double value);
bool genOutputs(int handle,ENUM_TRADE trade,int StopLoss,int TakeProfit,double duration);
bool genInputs(int handle);
bool setInputs(int handle,double &Inputs[],int nInputs);
bool setOutputs(int handle,bool &Outputs[]);
bool training(int handle);
bool classify(int handle);
bool classify(int handle,int offset);
bool classify(int handle,double &iput[]);
void deinitSVMachine(void);
#import
//---  ,     svm
int N_Inputs=7;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnStart()
  {
   double inputs[];           //   double,         
   bool   outputs[];          //   bool,         
   int N_TrainingPoints=5000; //    
   int N_TestPoints=5000;     //   ,     

   genTrainingData(inputs,outputs,N_TrainingPoints); //    ,      svm

   int handle1=initSVMachine();             //        
   setInputs(handle1,inputs,7);             //   ( )     
   setOutputs(handle1,outputs);             //   ( )     
   setParameter(handle1,OP_TOLERANCE,0.01); //    <5%
   training(handle1);                       //      /  

   insertRandomErrors(inputs,outputs,500);  //  /      

   int handle2=initSVMachine();             //        
   setInputs(handle2,inputs,7);             //   ( )     
   setOutputs(handle2,outputs);             //   ( )     
   setParameter(handle2,OP_TOLERANCE,0.01); //    <5%
   training(handle2);                       //      /  

   double t1=testSVM(handle1,N_TestPoints); //         t1
   double t2=testSVM(handle2,N_TestPoints); //         t2

   Print(" SVM  ",NormalizeDouble(t1,2),"% (    )");
   Print(" SVM  ",NormalizeDouble(t2,2),"% (    )");
   deinitSVMachine(); //  ,    SVM    
  }
//+------------------------------------------------------------------+
//|        
//|  ,   ,  true/false 
//|    ,    ,  
//+------------------------------------------------------------------+
bool isItASchnick(double height,double weight,double N_legs,double N_eyes,double L_arm,double av_speed,double f_call)
  {
   if(height   < 1000  || height   > 1100)  return(false); //      > return(false)
   if(weight   < 40    || weight   > 50)    return(false); //      > return(false)
   if(N_legs   < 8     || N_legs   > 10)    return(false); //  N_Legs     > return(false)
   if(N_eyes   < 3     || N_eyes   > 4)     return(false); //  the N_eyes     > return(false)
   if(L_arm    < 400   || L_arm    > 450)   return(false); //  L_arm      > return(false)
   if(av_speed < 2     || av_speed > 2.5)   return(false); //  av_speed     > return(false)
   if(f_call   < 11000 || f_call   > 15000) return(false); //  f_call     > return(false)
   return(true);                                           //   > return(true)
  }
//+------------------------------------------------------------------+
//|       double  bool, 
//|  / ,    SVM
//+------------------------------------------------------------------+ 
void genTrainingData(double &inputs[],bool &outputs[],int N)
  {
   double in[];                    //    double    
   ArrayResize(in,N_Inputs);       //   in[]  N_Inputs
   ArrayResize(inputs,N*N_Inputs); //   inputs[]  N*N_Inputs 
   ArrayResize(outputs,N);         //  outputs[]  N 
   for(int i=0;i<N;i++)
     {
      in[0]=    randBetween(980,1120);    //    
      in[1]=    randBetween(38,52);       //    
      in[2]=    randBetween(7,11);        //    N_legs
      in[3]=    randBetween(3,4.2);       //    N_eyes
      in[4]=    randBetween(380,450);     //    L_arms
      in[5]=    randBetween(2,2.6);       //    av_speed
      in[6]=    randBetween(10500,15500); //    f_call
      
      //---         
      ArrayCopy(inputs,in,i*N_Inputs,0,N_Inputs);
      //---     ,    
      outputs[i]=isItASchnick(in[0],in[1],in[2],in[3],in[4],in[5],in[6]);
     }
  }
//+------------------------------------------------------------------+
//|      SVM    
//|      
//+------------------------------------------------------------------+ 
double testSVM(int handle,int N)
  {
   double in[];
   int atrue=0;
   int afalse=0;
   int N_correct=0;
   bool Predicted_Output;
   bool Actual_Output;
   ArrayResize(in,N_Inputs);
   for(int i=0;i<N;i++)
     {
      in[0]=    randBetween(980,1120);    //    
      in[1]=    randBetween(38,52);       //    
      in[2]=    randBetween(7,11);        //    N_legs
      in[3]=    randBetween(3,4.2);       //    N_eyes
      in[4]=    randBetween(380,450);     //    L_arms
      in[5]=    randBetween(2,2.6);       //    av_speed
      in[6]=    randBetween(10500,15500); //    f_call
      
      //---   isItASchnick     
      Actual_Output=isItASchnick(in[0],in[1],in[2],in[3],in[4],in[5],in[6]);
      //---   SVM   .
      Predicted_Output=classify(handle,in);
      if(Actual_Output==Predicted_Output)
        {
         N_correct++; //     .
        }
     }
//---    SVM  
   return(100*((double)N_correct/(double)N));
  }
//+------------------------------------------------------------------+
//|         
//|     N    
//+------------------------------------------------------------------+ 
void insertRandomErrors(double &inputs[],bool &outputs[],int N)
  {
   int nTrainingPoints=ArraySize(outputs); //   
   int index;                              //   'index'
   bool randomOutput;                      //   'randomOutput'  'bool'
   double in[];                            //    double    
   ArrayResize(in,N_Inputs);               //   in[]  N_Inputs
   for(int i=0;i<N;i++)
     {
      in[0]=    randBetween(980,1120);    //    
      in[1]=    randBetween(38,52);       //    
      in[2]=    randBetween(7,11);        //    N_legs
      in[3]=    randBetween(3,4.2);       //    N_eyes
      in[4]=    randBetween(380,450);     //    L_arms
      in[5]=    randBetween(2,2.6);       //    av_speed
      in[6]=    randBetween(10500,15500); //    f_call
      
      //---            
      index=(int)MathRound(randBetween(0,nTrainingPoints-1));
      //---     bool   
      if(randBetween(0,1)>0.5) randomOutput=true;
      else                     randomOutput=false;
      
      //---         
      ArrayCopy(inputs,in,index*N_Inputs,0,N_Inputs);
      //---         
      outputs[index]=randomOutput;
     }
  }
//+------------------------------------------------------------------+
//|         
//| t1  t2
//+------------------------------------------------------------------+ 
double randBetween(double t1,double t2)
  {
   return((t2-t1)*((double)MathRand()/(double)32767)+t1);
  }
//+------------------------------------------------------------------+
